@@ -23,6 +23,14 @@
 from __future__ import print_function
 import base64
 import requests
+try:
+    # Python 3
+    from urllib import parse as urlparse
+    from json.decoder import JSONDecodeError
+except ImportError:
+    # Python 2
+    import urlparse
+    JSONDecodeError = ValueError
 
 
 class Helpers(object):
@@ -121,6 +129,64 @@ class ApiObject(object):
             self.tracker.reset()
         return self.compute_url()
 
+    @staticmethod
+    def collate_pages(get_request, data):
+        """Given a GET request whose results span across multiple pages, crawl
+        each page and collate the results.
+
+        :param first_page_data: "results" dict for first pagefull of dat
+        :type first_page_data: dict
+        :param request: Request used to get first page
+        :type request: requests.PreparedRequest
+        """
+        # First, sanity check that all is good. Only process GET requests:
+        if get_request.method.upper().strip() != "GET":
+            return data
+
+        # Only attempt to pull subsequent pages if we can verify that there are
+        # subsequent pages "to be pulled"...
+        if isinstance(data, dict) and "results" in data.keys():
+            collated_data = data["results"]
+            while "nextPage" in data.keys() and data["nextPage"]:
+                # Get the next page full of data.
+                next_page = requests.get(
+                    get_request.url,
+                    params={'pageNo': int(data['pageNo']) + 1},
+                    headers=get_request.headers
+                )
+
+                if not next_page.ok:
+                    # Return an empty result.
+                    collated_data = []
+                    break
+
+                data = next_page.json()
+                collated_data = collated_data + data['results']
+                del next_page
+
+            # Dismantle the URL to see if data was requested in descending
+            # order...
+            query_params = dict(
+                urlparse.parse_qsl(urlparse.urlsplit(get_request.url).query)
+            )
+            descending_order = 'isDescendingOrder' in query_params.keys() and \
+                query_params['isDescendingOrder']
+
+            # Re-create the final data set as if it were a single page
+            # containing all records to ensure that existing stuff that expects
+            # this data structure doesn't fall over.
+            return {
+                'results': collated_data,
+                'totalResults': len(collated_data),
+                'pageNo': 1,
+                'pageSize': len(collated_data),
+                'nextPage': False,
+                'previousPageNo': 0,
+                'descendingOrder': descending_order
+            }
+        else:
+            return data
+
     def compute_url(self, suffix=''):
         retval = self.baseurl
         suffix = self.tracker.fullpath(suffix)
@@ -147,8 +213,17 @@ class ApiObject(object):
             )
             raise RuntimeError(msg)
         try:
-            return resp.json()
-        except: # noqa
+            data = resp.json()
+            # Some GET requests return paginated output. If all the data fits
+            # in one page, return just the contents of the "results" list,
+            # otherwise, we need to do an assembly job to collate the entire
+            # list of results from all pages and return the full list.
+            if resp.request.method == "GET" and isinstance(data, dict) and \
+                    data.get("nextPage", None):
+                return ApiObject.collate_pages(resp.request, data=data)
+            else:
+                return data
+        except JSONDecodeError:
             return resp.text
 
     def get(self, suffix='', headers={}):
@@ -169,10 +244,10 @@ class ApiObject(object):
         resp = requests.put(url, headers=hdr, data=data, json=json)
         return self.process_result(url, resp)
 
-    def delete(self, suffix='', headers={}):
+    def delete(self, suffix='', headers={}, data=None, json=None):
         url = self.compute_url(suffix)
         hdr = self.prep_headers(headers)
-        resp = requests.delete(url, headers=hdr)
+        resp = requests.delete(url, headers=hdr, data=data, json=json)
         return self.process_result(url, resp)
 
 
